/*
        +==========================================================+
        |                                                          |
        |           Perl extension for NDS* and Netware*           |
        |           --------------------------------               |
        |           Buf_T.xs : Implements all NDS retrieve/put.    |
        |                                                          |
        |            by Steinar Kleven <stk@ahs.hist.no>           |
        |                                                          |
        |                  version 1.0                             |
        |                                                          |
        |                                                          |
        | Copyright (c) 1997 Steinar Kleven. All rights reserved.  |
        |   This program is NOT free software, you can NOT         |
        |   redistribute it and/or modify it.                      |
        |                                                          |
        |  * NDS and Netware are trademarks of Novell Inc.         |
        |                                                          |
        +==========================================================+
*/




/* Turn of these annoying "Structure packing size .." Warnings */
#if defined(__BORLANDC__)
        #pragma warn -pck
#endif


#ifdef NLM_PLATFORM
  #define N_PLAT_NETWARE
  #define N_PLAT_NLM
  #define N_ARCH_32
  #ifndef LC_ALL
    #define LC_ALL 0
  #endif
  #undef _Windows
  void *my_fdopen(int , char*);
  extern void* gpf_fdopen;
  #pragma aux my_fdopen = "call gpf_fdopen"
#endif


#define MYLOWORD(l)           ((nuint16)(l))
#define MYHIWORD(l)           ((nuint16)(((nuint32)(l) >> 16) & 0xFFFF))

#if !defined(N_PLAT_NLM)
  #define WIN32_LEAN_AND_MEAN
  #include <windows.h>
#endif // !N_PLAT_NLM

#include <fcntl.h>
#include <stdio.h>

/* Novell SDK includes */
#ifdef __cplusplus
extern "C" {
#endif
#include "nwnet.h"
#include "nwcalls.h"
#include "nwlocale.h"
#ifdef __cplusplus
}
#endif


#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#undef _open_osfhandle


/* #include <io.h> */


/* PerlWare headers */
#include "PerlWare.h"


typedef struct _replica_type
{
   nuint16 s1;
   nuint16 s2;
} replica_type;



/* ErrCode is set to the return value of last NW* called.
   Users creating scripts based on this extension can
   retrieve this by calling LastErr at class the error aqured.
*/



NWDSCCODE N_API ErrCode = 0;



#define CONV_SIZE 8192


/* If you want to use the internal debugging you should define this.
   You can comment it out if you want this extension to go 1 ns. faster. (whitout
   possibilities of turning it on.
*/

// The different debug types OR'ed


#define DBG_ARGS 0x1
#define DBG_NWRET 0x2
#define DBG_DUMP 0x4
#define DBG_CONV 0x8
// #define NDSM_DBG (DBG_CONV|DBG_ARGS|DBG_NWRET|DBG_DUMP)
#undef NDSM_DBG
#if (NDSM_DBG > 0)
#define DBGMSG(_x_)  if (_dbglvl _x_){ printf("%s, %d: ",__FILE__,__LINE__); \
                     _dbgprint _x_; }
#else
#define DBGMSG(_x_)
#endif






/* This section handles messages of different sorts */

/* Debug, Errors aso */

#if (NDSM_DBG > 0)
bool _dbglvl(int type, char *format, ...)
{
/*   va_list ap;
   va_start(ap, format);
   va_end(ap);  */
 return (type && NDSM_DBG);
}


int _dbgprint(int type, char *format, ...)
{
   va_list ap;
   int cnt;
   va_start(ap, type);
   cnt = va_arg(ap, int);

   if (NDSM_DBG & type)
   {
      cnt = vprintf(format, ap);
   }
   va_end(ap);
   return(cnt);
}
#endif



/* This section handles conversion from human readable data to C-struct and back
   for datatypes defined in "Attribute Syntax Definitions" (NWSDK) */

nuint WhatType(nuint32 syntaxID)
{
   nuint Retval;
   switch (syntaxID)
   {
        case SYN_UNKNOWN:
      case SYN_CI_LIST:
      case SYN_OCTET_STRING:
      case SYN_FAX_NUMBER:
      case SYN_NET_ADDRESS:
      case SYN_OCTET_LIST:
      case SYN_EMAIL_ADDRESS:
      case SYN_PATH:
      case SYN_REPLICA_POINTER:
      case SYN_OBJECT_ACL:
      case SYN_PO_ADDRESS:
      case SYN_TIMESTAMP:
      case SYN_BACK_LINK:
      case SYN_TYPED_NAME:
      case SYN_HOLD:
        Retval = STRUCTTYPE;
      break;
      case SYN_STREAM:
        Retval = STREAMTYPE;
      break;
      case SYN_DIST_NAME:
      case SYN_CE_STRING:
      case SYN_CI_STRING:
      case SYN_PR_STRING:
      case SYN_NU_STRING:
      case SYN_TEL_NUMBER:
      case SYN_CLASS_NAME:
        Retval = STRINGTYPE;
      break;
      case SYN_BOOLEAN:
      case SYN_INTEGER:
      case SYN_COUNTER:
      case SYN_INTERVAL:
      case SYN_TIME:
            Retval = INTEGERTYPE;
      break;
/*      case SYN_COMPONENT:
#        case SYN_ENUM_COMPONENT:
#        case SYN_NULL:
#        case SYN_OID:
#        case SYN_ENUMERATED:
#        case SYN_SET:
#        case SYN_SEQUENCE:
#        case SYN_CHOICE:
#        case SYN_ANY:
        case SYNTAX_COUNT: */
   default:
        Retval = UNKNOWNTYPE;
      break;
   }
   return Retval;
}




#/* Caller is responsible for freeing the memory allocated here */
void * Human2Struct(PProto_ char *attrVal, nuint32 syntaxID, void *pBuf)
{
   unsigned long ulHash = 0;
   nuint i;
   void *hlpptr;
   nuint count;
   char *nextElem;
   char *hlpstr;

/*
        Structures converted here :
        I_List_T;
        Octet_String_T;
        Fax_Number_T;
        Octet_List_T;
        EMail_Address_T;
        Path_T;
        Replica_Pointer_T;
        Object_ACL_T;
        NWDS_TimeStamp_T;
        Stream_T;
        Back_Link_T;
        Typed_Name_T;
        Hold_T;
*/
  if (attrVal)
        {
   nextElem = strtok(attrVal, "\n");
        switch (syntaxID)
                {
                case SYN_UNKNOWN:
                break;
                case SYN_CI_LIST:
                  Newc(0, pBuf, 1, CI_List_T, CI_List_T);
                  hlpptr = pBuf;
                  while(nextElem)
                  {
                         ((CI_List_T *)hlpptr)->s = nextElem;
                         hlpptr = ((CI_List_T *)hlpptr)->next;
                         nextElem = strtok(NULL, "\n");
                         if (nextElem){
                                Newc(0, hlpptr, 1, CI_List_T, CI_List_T);
                         }
                  }
                break;
                case SYN_OCTET_STRING:
                  New(0, pBuf, 1, Octet_String_T);
                  sscanf(nextElem, "%lu\n", &((Octet_String_T *)pBuf)->length);
                  New(0, ((Octet_String_T *)pBuf)->data, ((Octet_String_T *)pBuf)->length, unsigned char);
                  nextElem = strtok(NULL, "\n");
                  count = 0;
                  for (i=0; i<((Octet_String_T *)pBuf)->length && nextElem; i++)
                  {
                         sscanf(nextElem, "%X", &((Octet_String_T *)pBuf)->data[count++]);
                         nextElem = strtok(NULL, "\n");
                  }
                  break;
                case SYN_FAX_NUMBER:
/* This is not complete, a bitstring is needed */
                  New(0, pBuf, 1, Fax_Number_T);
                  ((Fax_Number_T *)pBuf)->telephoneNumber = nextElem;
                  break;
                case SYN_NET_ADDRESS:
                  New(0, pBuf, 1, Net_Address_T);
                  sscanf(nextElem, "%lu", &((Net_Address_T *)pBuf)->addressType);
                  nextElem = strtok(NULL, "\n");
                  sscanf(nextElem, "%lu", &((Net_Address_T *)pBuf)->addressLength);
                  nextElem = strtok(NULL, "\n");
                  New(0, ((Net_Address_T *)pBuf)->address, ((Net_Address_T *)pBuf)->addressLength, NWNET_ADDR);
                  sscanf(nextElem, "%02X%02X%02X%02X:%02X%02X%02X%02X%02X%02X",
                                &((Net_Address_T *)pBuf)->address[0],
                                &((Net_Address_T *)pBuf)->address[1],
                                &((Net_Address_T *)pBuf)->address[2],
                                &((Net_Address_T *)pBuf)->address[3],
                                &((Net_Address_T *)pBuf)->address[4],
                                &((Net_Address_T *)pBuf)->address[5],
                                &((Net_Address_T *)pBuf)->address[6],
                                &((Net_Address_T *)pBuf)->address[7],
                                &((Net_Address_T *)pBuf)->address[8],
                                &((Net_Address_T *)pBuf)->address[9]);
                  break;
                case SYN_OCTET_LIST:
                  Newc(0, pBuf, 1, Octet_List_T, Octet_List_T);
                break;
                case SYN_EMAIL_ADDRESS:
                  New(0, pBuf, 1, EMail_Address_T);
                  sscanf(nextElem, "%lu", &(((EMail_Address_T *)pBuf)->type));
                  nextElem = strtok(NULL, "\n");
                  ((Net_Address_T *)pBuf)->address = (unsigned char*)nextElem;
                break;
                case SYN_PATH:
                  New(0, pBuf, 1, Path_T);
                  sscanf(nextElem, "%lu", &(((Path_T *)pBuf)->nameSpaceType));
                  nextElem = strtok(NULL, "\n");
                  ((Path_T *)pBuf)->volumeName = nextElem;
                  nextElem = strtok(NULL, "\n");
                  ((Path_T *)pBuf)->path = nextElem;
                DBGMSG((DBG_CONV, "SYN_PATH -> Struct : %d, %s, %s\n", ((Path_T *)pBuf)->nameSpaceType,
                                        ((Path_T *)pBuf)->volumeName, ((Path_T *)pBuf)->path = nextElem));
                break;
                case SYN_REPLICA_POINTER:
                  New(0, pBuf, 1, Replica_Pointer_T);
                  ((Replica_Pointer_T *)pBuf)->serverName = nextElem;
        nextElem = strtok(NULL, "\n");
        sscanf(nextElem, "%lu", &((Replica_Pointer_T *)pBuf)->replicaType);
        nextElem = strtok(NULL, "\n");
        sscanf(nextElem, "%u", &((Replica_Pointer_T *)pBuf)->replicaNumber);
                  nextElem = strtok(NULL, "\n");
        sscanf(nextElem, "%lu", &((Replica_Pointer_T *)pBuf)->count);
        nextElem = strtok(NULL, "\n");
//        New(0, ((Replica_Pointer_T *)pBuf)->replicaAddressHint[0], 1, Net_Address_T);
        sscanf(nextElem, "%lu", &(((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].addressLength));
        nextElem = strtok(NULL, "\n");
        sscanf(nextElem, "%lu\n%02X%02X%02X%02X:%02X%02X%02X%02X%02X%02X",
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].addressType,
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[0],
                                &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[1],
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[2],
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[3],
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[4],
                                &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[5],
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[6],
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[7],
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[8],
            &((Replica_Pointer_T *)pBuf)->replicaAddressHint[0].address[9]);
      break;
      case SYN_OBJECT_ACL:
                  Newc(0, pBuf, 1, Object_ACL_T, Object_ACL_T);
                  ((pObject_ACL_T)pBuf)->protectedAttrName = nextElem;
                  hlpstr = strtok(NULL, "\n");
                  ((pObject_ACL_T)pBuf)->subjectName = strtok(NULL, "\n");
                  ((pObject_ACL_T)pBuf)->privileges =
                          Str2ACL(((pObject_ACL_T)pBuf)->protectedAttrName, hlpstr);

                break;
                case SYN_PO_ADDRESS:
                  Newc(0, pBuf, 1, Postal_Address_T, Postal_Address_T);
                  count = 0;
                  while(nextElem && count < 6)
                  {
                         (*(Postal_Address_T *)pBuf)[count++] = (pnstr8)nextElem;
                         nextElem = strtok(NULL, "\n");
                  }
      break;
                case SYN_TIMESTAMP:
        Newc(0, pBuf, 1, NWDS_TimeStamp_T, pNWDS_TimeStamp_T);
        sscanf(nextElem, "%lu", &((NWDS_TimeStamp_T *)pBuf)->wholeSeconds);
        nextElem = strtok(NULL, "\n");
        sscanf(nextElem, "%lu", &((NWDS_TimeStamp_T *)pBuf)->eventID);
      break;
                case SYN_STREAM:
                  Newc(0, pBuf, 1, char, char*);
                  croak("You'll have to use OpenStream to write SYN_STREAM\n");
                break;
                case SYN_BACK_LINK:
                  Newc(0, pBuf, 1, Back_Link_T, Back_Link_T);
                  sscanf(nextElem, "%lu", &((Back_Link_T *)pBuf)->remoteID);
                  nextElem = strtok(NULL, "\n");
                  ((Back_Link_T *)pBuf)->objectName = nextElem;
      break;
      case SYN_TYPED_NAME:
        Newc(0, pBuf, 1, Typed_Name_T, Typed_Name_T);
        ((Typed_Name_T *)pBuf)->objectName = nextElem;
        nextElem = strtok(NULL, "\n");
        sscanf(nextElem, "0x%x", &((Typed_Name_T *)pBuf)->level);
        nextElem = strtok(NULL, "\n");
                  sscanf(nextElem, "0x%x", &((Typed_Name_T *)pBuf)->interval);
      break;
      case SYN_HOLD:
        Newc(0, pBuf, 1, Hold_T, Hold_T);
        ((Hold_T *)pBuf)->objectName = nextElem;
        nextElem = strtok(NULL, "\n");
        sprintf(nextElem, "%lu", &((Hold_T *)pBuf)->amount);
      break;
                default:
      break;
      };
   }
        return pBuf;
}




/* Decode the reights from bit-defs to string */
void ACL2Str(pnstr8 attrName, pnstr8 rightsstring, nuint32 privs)
{
        strcpy(rightsstring,"[");
        if(!strcmp(attrName, "[Entry Rights]"))
        {
                if(privs & 0x01)                                /* DS_ENTRY_BROWSE */
                        strcat(rightsstring, "B");
                if(privs & 0x02)                                /* DS_ENTRY_ADD */
                        strcat(rightsstring, "A");
                if(privs & 0x04)                                /* DS_ENTRY_DELETE */
                        strcat(rightsstring, "D");
                if(privs & 0x08)                                /* DS_ENTRY_RENAME */
                        strcat(rightsstring, "R");
                if(privs & 0x10)                                /* DS_ENTRY_SUPERVISOR */
                        strcat(rightsstring, "S");
        }
        else
        {
                if(!strcmp(attrName, "[SMS Rights]"))
                {
                        if(privs & 0x01)                                        /* DS_SMS_SCAN */
                                strcat(rightsstring, "S");
                        if(privs & 0x02)                                        /* DS_SMS_BACKUP */
                                strcat(rightsstring, "B");
                        if(privs & 0x04)                                        /* DS_SMS_RESTORE */
                                strcat(rightsstring, "R");
                        if(privs & 0x08)                                        /* DS_SMS_RENAME */
                                strcat(rightsstring, "C");
                        if(privs & 0x10)                                        /* DS_SMS_DELETE */
                                strcat(rightsstring, "D");
                        if(privs & 0x20)                                        /* DS_SMS_ADMIN */
                                strcat(rightsstring, "A");
                }
                else    /* Attribute Rights which applies to all others */
                {
                        if(privs & 0x01)                                        /* DS_ATTR_COMPARE */
                                strcat(rightsstring, "C");
                        if(privs & 0x02)                                        /* DS_ATTR_READ */
                                strcat(rightsstring, "R");
                        if(privs & 0x04)                                        /* DS_ATTR_WRITE */
                                strcat(rightsstring, "W");
                        if(privs & 0x08)                                        /* DS_ATTR_SELF */
                                strcat(rightsstring, "A");
                        if(privs & 0x20)                                        /* DS_ATTR_SUPERVISOR */
                                strcat(rightsstring, "S");
                }
        }
        strcat(rightsstring,"]\x0");
}



char *Struct2Human(void *pValue, nuint syntaxID, char *genbuf)
{

/* Begin helper vars */
        int i;
   char helpstr[256] = "";
//   uint32 hlp32;
/* End helper vars  */
/* check if we got some data, we don't want to read from NULL. */
        if (pValue)
        {
      switch (syntaxID)
      {
        case SYN_UNKNOWN:
        break;
      case SYN_CI_LIST:
         *genbuf = 0;
         do {
               sprintf(helpstr, "%s\n", ((CI_List_T *)pValue)->s);
             strcat(genbuf, helpstr);
             pValue = ((CI_List_T *)pValue)->next;
        }while (pValue);
      break;
      case SYN_OCTET_STRING:
      DBGMSG((DBG_CONV, "Going to decode SYN_OCTET_STRING at address: %p\n", pValue));
      DBGMSG((DBG_CONV, "SYN_OCTET_STRING->length: %d\n",((Octet_String_T *)pValue)->length));
//      exit(0);

     sprintf(genbuf, "%lu\n",((Octet_String_T *)pValue)->length);
          for(i=0; i<((Octet_String_T *)pValue)->length && i < CONV_SIZE; i++)
          {
          sprintf(helpstr, "%X\n",(nuint8)((Octet_String_T *)pValue)->data[i]);
          strcat(genbuf, helpstr);
          }
        break;
      case SYN_FAX_NUMBER:
/* SYN_FAX_NUMBER also have a bitstring */
        sprintf(genbuf, "%s\n",((Fax_Number_T *)pValue)->telephoneNumber);
        break;
      case SYN_NET_ADDRESS:
        sprintf(genbuf, "%lu\n%lu\n%02X%02X%02X%02X:%02X%02X%02X%02X%02X%02X\n",
                                ((Net_Address_T *)pValue)->addressType,
                                ((Net_Address_T *)pValue)->addressLength,
                                ((Net_Address_T *)pValue)->address[0],
                                ((Net_Address_T *)pValue)->address[1],
                                ((Net_Address_T *)pValue)->address[2],
                                ((Net_Address_T *)pValue)->address[3],
                                ((Net_Address_T *)pValue)->address[4],
                                ((Net_Address_T *)pValue)->address[5],
                                ((Net_Address_T *)pValue)->address[6],
                                ((Net_Address_T *)pValue)->address[7],
                                ((Net_Address_T *)pValue)->address[8],
                                ((Net_Address_T *)pValue)->address[9]);
        break;
      case SYN_OCTET_LIST:
        *genbuf = 0;
      break;
      case SYN_EMAIL_ADDRESS:
        sprintf(genbuf, "%lu\n%s\n", ((EMail_Address_T *)pValue)->type, ((EMail_Address_T *)pValue)->address);
      break;
      case SYN_PATH:
                sprintf(genbuf, "%lu\n%s\n%s\n",((Path_T *)pValue)->nameSpaceType,
                                         ((Path_T *)pValue)->volumeName,
                                         ((Path_T *)pValue)->path);
      DBGMSG((DBG_DUMP, "SYN_PATH -> char*: %s\n", genbuf));
      break;
      case SYN_REPLICA_POINTER:
      sprintf(genbuf, "%s\n%hu\n%hu\n%lu\n%lu", ((Replica_Pointer_T *)pValue)->serverName,
//                *((pnuint8)(&((Replica_Pointer_T *)pValue)->replicaType)),
//                *((pnuint8)(&((Replica_Pointer_T *)pValue)->replicaType)+2),
                                         MYLOWORD (((Replica_Pointer_T *)pValue)->replicaType),
                MYHIWORD (((Replica_Pointer_T *)pValue)->replicaType),
                                         ((Replica_Pointer_T *)pValue)->replicaNumber,
                                         ((Replica_Pointer_T *)pValue)->count);
      for(i=0; i < ((Replica_Pointer_T *)pValue)->count; i++)
        {
   if (((Replica_Pointer_T *)pValue)->replicaAddressHint[i].addressType == 9)
   {
     sprintf(helpstr, "%lu\n%u.%u.%u.%u\n",  ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].addressType,
//     ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[0],
//          ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[1],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[2],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[3],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[4],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[5],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[6],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[7]);
//          ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[8],
//          ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[9]);

   }else{
     sprintf(helpstr, "%lu\n%02X%02X%02X%02X:%02X%02X%02X%02X%02X%02X\n",  ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].addressType,
       ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[0],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[1],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[2],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[3],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[4],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[5],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[6],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[7],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[8],
            ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].address[9]);
   }
   DBGMSG((DBG_DUMP, "SYN_REPLICA_POINTER(ADR): Type:%lu, Length:%lu\n", ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].addressType,
                     ((Replica_Pointer_T *)pValue)->replicaAddressHint[i].addressLength));
   DBGMSG((DBG_DUMP, "ReplicaState: %p: %X, %X\n", &((Replica_Pointer_T *)pValue)->replicaType, 
                                                        ((replica_type*)&(((Replica_Pointer_T *)pValue)->replicaType))->s1, 
                                                        ((replica_type*)&(((Replica_Pointer_T *)pValue)->replicaType))->s2));
        strcat(genbuf, helpstr);
       }
      break;
      case SYN_OBJECT_ACL:
                  ACL2Str(((Object_ACL_T *)pValue)->protectedAttrName, helpstr, ((Object_ACL_T *)pValue)->privileges);
                  sprintf(genbuf, "%s\n%s\n%s\n", ((Object_ACL_T *)pValue)->protectedAttrName, helpstr,
                                                        ((Object_ACL_T *)pValue)->subjectName);
      break;
      case SYN_PO_ADDRESS:
        sprintf(genbuf, "%s\n%s\n%s\n%s\n%s\n%s\n",((pnstr8 *)pValue)[0],((pnstr8 *)pValue)[1],
                                                ((pnstr8 *)pValue)[2],((pnstr8 *)pValue)[3],
                                                ((pnstr8 *)pValue)[4],((pnstr8 *)pValue)[5]);
      break;
      case SYN_TIMESTAMP:
        sprintf(genbuf, "%lu\n%lu\n", ((NWDS_TimeStamp_T *)pValue)->wholeSeconds, ((NWDS_TimeStamp_T *)pValue)->eventID);
   DBGMSG((DBG_DUMP, "Struct2Human SYN_TIMESTAMP %lu, %lu\n", ((NWDS_TimeStamp_T *)pValue)->wholeSeconds,
                        ((NWDS_TimeStamp_T *)pValue)->eventID));
      break;
      case SYN_STREAM:
/* Not supported, user should call $genbuf->OpenStream() */
      *genbuf = 0;
      break;
      case SYN_BACK_LINK:
        sprintf(genbuf, "%lu\n%s\n", ((Back_Link_T *)pValue)->remoteID,((Back_Link_T *)pValue)->objectName);
      break;
      case SYN_TYPED_NAME:
        sprintf(genbuf, "0x%x\n0x%x\n0x%x\n", ((Typed_Name_T *)pValue)->objectName,
                                           ((Typed_Name_T *)pValue)->level,
                                           ((Typed_Name_T *)pValue)->interval);
      break;
      case SYN_HOLD:
          sprintf(genbuf, "%s\n%lu\n",((Hold_T *)pValue)->objectName,((Hold_T *)pValue)->amount);
      break;
      default:
      break;
      };
   }
   return genbuf;
}


/* Used to free ANY of the structs above */
void SafefreeStruct(PProto_ void *ptr, nuint32 syntaxID)
{
  void *hlpptr = ptr;
  switch (syntaxID)
  {
    case SYN_CI_LIST:
      while(((CI_List_T *)ptr)->next)
      {
        hlpptr = ((CI_List_T *)ptr)->next;
        Safefree(ptr);
        ptr = ((CI_List_T *)hlpptr);
      }
    case SYN_OCTET_STRING:
      Safefree(((Octet_String_T *)ptr)->data);
    break;
    case SYN_NET_ADDRESS:
      Safefree(((Net_Address_T *)ptr)->address);
    break;
    case SYN_REPLICA_POINTER:
      Safefree(((Replica_Pointer_T *)ptr)->replicaAddressHint);
    break;

  }
  Safefree(ptr);

}


MODULE = PerlWare::Buf_T  PACKAGE = PerlWare::Buf_T  PREFIX = ndsm_
PROTOTYPES: ENABLE



long
ndsm_constant(name, arg)
        char *          name
        int             arg
        CODE:
                RETVAL = constant(name, arg);
    OUTPUT:
                RETVAL

#/* Returns the last ErrCode */
long
ndsm_GetLastErr()
        CODE:
      RETVAL = ErrCode;
        OUTPUT:
      RETVAL


#/* Set the registration info in .LIB */
void
ndsm_SetRegInfo(Key, Code)
                char *Key;
      long Code;
        CODE:
           libSetRegInfo(Key, Code);



#/* Implemented in Buf_T.pm      */
void
ndsm_NWDSAddObject(NDScontext, objectName, iterationHandle, more, objectInfo)
                NWDSContextHandle   NDScontext
                pnstr8              objectName
                nint32              iterationHandle
                nbool8              more
                pBuf_T              objectInfo
        PPCODE:
                DBGMSG((DBG_ARGS, "XS_NWDSAddObject called with: (%d, %s, %d, %d, %p)\n",
                                                NDScontext, objectName, iterationHandle, more, objectInfo));
                ErrCode = NWDSAddObject(NDScontext, objectName, &iterationHandle, more, objectInfo);
                DBGMSG((DBG_NWRET, "NWDSAddObject returned code: %X\n", ErrCode));
                EXTEND(sp, 1);
                if (!ErrCode){
                  PUSHs(sv_2mortal((SV*)newSViv(iterationHandle)));
                }else{
                  PUSHs(&sv_undef);
                }



#/* Implemented in Buf_T                */
#/* Obsolete  9/97 */
void
ndsm_NWDSAddPartition(NDScontext, server, partitionRoot, iterationHandle, more, objectInfo)
                NWDSContextHandle   NDScontext
                pnstr8              server
                pnstr8              partitionRoot
                nint32              iterationHandle
                nbool8              more
                pBuf_T              objectInfo
        PPCODE:
                ErrCode = NWDSAddPartition(NDScontext, server, partitionRoot, &iterationHandle, more, objectInfo);
                EXTEND(sp, 1);
                if (!ErrCode){
                  PUSHs(sv_2mortal((SV*)newSViv(iterationHandle)));
                }else{
                  PUSHs(&sv_undef);
                }


#/* Implemented in Buf_T.pm             */
NWDSCCODE
ndsm_NWDSAllocBuf(size, pBuf)
                size_t size
                pBuf_T pBuf = NO_INIT
        CODE:
                RETVAL = NWDSAllocBuf(size, &pBuf);
                DBGMSG((DBG_NWRET, "XS_NWDSAllocBuf(): size = %d, pBuf = %p\n", size, pBuf));
        OUTPUT:
                pBuf
                RETVAL



#/* Not used in 1.0 */
# NWDSCCODE
# ndsm_NWDSBeginClassItem(NDScontext, genbuf)
#                 NWDSContextHandle   NDScontext
#                pBuf_T              genbuf
#          CODE:
#                RETVAL = NWDSBeginClassItem(NDScontext, genbuf);
#        OUTPUT:
#                RETVAL



NWDSCCODE
ndsm_NWDSCloseIteration(NDScontext, iterationHandle, operation)
                NWDSContextHandle       NDScontext
                int32                   iterationHandle
                uint32                  operation
        CODE:
                RETVAL = NWDSCloseIteration(NDScontext, iterationHandle, operation);
        OUTPUT:
                RETVAL


#/* Implemented in Buf_T.pm             */
void
ndsm_NWDSCompare(NDScontext, objectName, genbuf)
                NWDSContextHandle   NDScontext
                pnstr8              objectName
                pBuf_T              genbuf
        PREINIT:
                nbool8              matched;
        PPCODE:
                ErrCode = NWDSCompare(NDScontext, objectName, genbuf, &matched);
                EXTEND(sp, 1);
                if (!ErrCode){
                  DBGMSG((DBG_DUMP, "NWDSCompare returned MATCH!\n"));
                  PUSHs(sv_2mortal((SV*)newSViv(matched)));
                }else{
                  PUSHs(&sv_undef);
                }


# /* Not used in 1.0 */
# NWDSCCODE
# ndsm_NWDSComputeAttrValSize(NDScontext, genbuf, syntaxID, attrValSize)
#                NWDSContextHandle   NDScontext
#                pBuf_T              genbuf
#                nuint32             syntaxID
#                U32                 attrValSize
#        CODE:
#                RETVAL = NWDSComputeAttrValSize(NDScontext, genbuf, syntaxID, &attrValSize);
#        OUTPUT:
#                attrValSize
#                RETVAL


                
#/* Implemented in Buf_T.pm     */
NWDSCCODE
ndsm_NWDSFreeBuf(pBuf)
                pBuf_T pBuf
        CODE:
                DBGMSG((DBG_DUMP, "NWDSFreeBuf(%p)\n", pBuf));
                RETVAL = NWDSFreeBuf(pBuf);
        OUTPUT:
                RETVAL



#/* Implemented in Buf_T.pm        */
void
ndsm_NWDSGetAttrCount(NDScontext, genbuf)
                NWDSContextHandle   NDScontext
                pBuf_T              genbuf
        PREINIT:
                U32               attrCount;
        PPCODE:
                ErrCode = NWDSGetAttrCount(NDScontext, genbuf, &attrCount);
                EXTEND(sp, 1);
                if (!ErrCode){
                  PUSHs(sv_2mortal((SV*)newSViv(attrCount)));
                }else{
                  PUSHs(&sv_undef);
                }


#/* ndsm_NWDSGetAttrDef() is not implemented */



#/* Implemented in Buf_T.pm        */
void
ndsm_NWDSGetAttrName(NDScontext, genbuf)
                NWDSContextHandle   NDScontext
                pBuf_T              genbuf
        PREINIT:
                pnstr8              attrName;
                U32                        attrValCount;
                U32                  syntaxID;
        PPCODE:
                Newz(0, attrName, MAX_DN_CHARS+1, char);
                ErrCode = NWDSGetAttrName(NDScontext, genbuf, attrName, &attrValCount, &syntaxID);
                DBGMSG((DBG_DUMP, "GettAttrName returned: %d, %d, %X, %s, %d, %d\n",
                        ErrCode, NDScontext, genbuf, attrName, attrValCount, syntaxID));
                if (!ErrCode){
                  EXTEND(sp, 3);
                  PUSHs(sv_2mortal((SV*)newSVpv(attrName, strlen(attrName))));
                  PUSHs(sv_2mortal((SV*)newSViv(attrValCount)));
                  PUSHs(sv_2mortal((SV*)newSViv(syntaxID)));
                }else{
/* Return nothing at all
                  EXTEND(sp, 1);
                  PUSHs(&sv_undef); */
                }
                Safefree(attrName);



int
ndsm_xsIsNumType(syntaxId)
                nuint32 syntaxId
        CODE:
                RETVAL = (WhatType(syntaxId) == INTEGERTYPE) ? TRUE : FALSE;
        OUTPUT:
                RETVAL

int
ndsm_xsIsStringType(syntaxId)
                nuint32 syntaxId
        CODE:
                RETVAL = (WhatType(syntaxId) == STRINGTYPE) ? TRUE : FALSE;
        OUTPUT:
                RETVAL

int
ndsm_xsIsMLType(syntaxId)
                nuint32 syntaxId
        CODE:
                RETVAL = (WhatType(syntaxId) == STRUCTTYPE) ? TRUE : FALSE;
        OUTPUT:
                RETVAL


# This is tricky...
# You should read the ndstypes.htm file to understand the en/de -coding of
# NDS values.
#/* Implemented in Buf_T.pm        */
void
ndsm_NWDSGetAttrVal(NDScontext, genbuf, syntaxID)
                NWDSContextHandle   NDScontext
                pBuf_T              genbuf
                nuint32             syntaxID
        PREINIT:
                nuint32             attrValSize;
      SV *                SvRetVal;
      void *              pVoidBuf;
      char *              HumanReadable;
      nint32              Integer;
        PPCODE:
      EXTEND(sp, 1);
      ErrCode = NWDSComputeAttrValSize(NDScontext, genbuf, syntaxID, &attrValSize);
      if (!ErrCode)
      {
        Newc(0, pVoidBuf, attrValSize, char, char*);
        ErrCode = NWDSGetAttrVal(NDScontext, genbuf, syntaxID, (nptr)pVoidBuf);
        if (!ErrCode)
        {
                         switch (WhatType(syntaxID))
                         {
                         case  INTEGERTYPE:
                                // Adjust for small integers (SYN_BOOLEAN)
                                Integer = (nint32)*((nint32*)pVoidBuf) & (nint32)(pow(2, (attrValSize * 8))-1);

                                DBGMSG((DBG_ARGS, "NWDSGetAttrVal: Called with SyntaxID = %d (NUMBER)\n", syntaxID));
                                DBGMSG((DBG_NWRET, "NWDSGetAttrVal: Number : %d, Size: %d\n", Integer, attrValSize));
                                if (syntaxID == SYN_TIME)
                                {
                                        HumanReadable = ctime((time_t*)pVoidBuf);
#                                        DBGMSG((DBG_DUMP, "_timezone = %d, _daylight=%d\n", _timezone, _daylight));
                                        DBGMSG((DBG_DUMP, "Converting time: ctime(%d) -> %s\n", (nint32)*((nint32*)pVoidBuf), HumanReadable));
                                        SvRetVal = sv_2mortal((SV*)newSViv((nint32)*((nint32*)pVoidBuf)));
                                }
                                else
                                        SvRetVal = sv_2mortal((SV*)newSViv( (nint32) Integer ));
                         break;
                         case STRINGTYPE:
                                DBGMSG((DBG_ARGS, "NWDSGetAttrVal: Called with SyntaxID=%d (STRING) \n", syntaxID));
                                DBGMSG((DBG_NWRET, "NWDSGetAttrVal: String : %s, Size: %d\n", (char*)pVoidBuf, attrValSize));
                                SvRetVal = sv_2mortal(newSVpv((char*)pVoidBuf, strlen((char*)pVoidBuf)));
                         break;
                         case STRUCTTYPE:
                                DBGMSG((DBG_ARGS, "NWDSGetAttrVal: Called with SyntaxID=%d (MLSTRING) \n", syntaxID));
                                Newc(0, HumanReadable, CONV_SIZE, char, char);
                                HumanReadable = Struct2Human(pVoidBuf, syntaxID, HumanReadable);
                                DBGMSG((DBG_NWRET, "NWDSGetAttrVal returning : %s, Size: %d\n", HumanReadable, strlen(HumanReadable)));
                                SvRetVal = sv_2mortal(newSVpv((nstr*)HumanReadable, strlen(HumanReadable)));
                                Safefree(HumanReadable);
                         break;
                         case STREAMTYPE:
# // For streams we just returns an empty string.
            SvRetVal = sv_2mortal(newSVpv((nstr*)"", strlen("")));
          break;
                         case UNKNOWNTYPE:
                         default:
                                warn("NWDSGetAttrVal() called with unknown syntaxID: %lu\n", syntaxID);
            SvRetVal = &sv_undef;
                         break;
                         }
                         PUSHs(SvRetVal);
                  }else{
                         PUSHs(&sv_undef);
                  }
                  Safefree(pVoidBuf);
                }else{
                  PUSHs(&sv_undef);
                }


#/* Implemented in Buf_T                        */
void
ndsm_NWDSGetObjectCount (NDScontext, genbuf)
                NWDSContextHandle   NDScontext;
                pBuf_T              genbuf;
        PREINIT:
          nuint32             objectCount;
        PPCODE:
                ErrCode = NWDSGetObjectCount (NDScontext, genbuf, &objectCount);
                EXTEND(sp, 1);
                if (!ErrCode)
                {
                  PUSHs(sv_2mortal((SV*)newSViv(objectCount)));
                }else{
                 PUSHs(&sv_undef);
                }



#/* Implemented in Buf_T.pm                     */
void
ndsm_NWDSGetObjectName (NDScontext, genbuf)
                NWDSContextHandle       NDScontext;
                pBuf_T                  genbuf;
        PREINIT:
                pnstr8                  objectName;
                nuint32                 attrCount;
                pObject_Info_T          pObjectInfo;
        PPCODE:
                Newz(0, objectName, 2*(MAX_DN_CHARS+1), nstr8);
                New(0, pObjectInfo, 1, Object_Info_T);
                ErrCode = NWDSGetObjectName(NDScontext, genbuf, objectName, &attrCount, pObjectInfo);
                DBGMSG((DBG_NWRET, "NWDSGetObjectName returned: %d, %X, %s, %d, %X\n", NDScontext, genbuf,
                                                                                                        objectName, attrCount, pObjectInfo));
                if (!ErrCode)
                {
                  EXTEND(sp, 6);
                  PUSHs(sv_2mortal((SV*)newSVpv(objectName, strlen(objectName))));
                  PUSHs(sv_2mortal((SV*)newSViv(attrCount)));
        PUSHs(sv_2mortal((SV*)newSViv(pObjectInfo->objectFlags)));
        PUSHs(sv_2mortal((SV*)newSViv(pObjectInfo->subordinateCount)));
        PUSHs(sv_2mortal((SV*)newSViv(pObjectInfo->modificationTime)));
                  PUSHs(sv_2mortal((SV*)newSVpv(pObjectInfo->baseClass, strlen(pObjectInfo->baseClass))));
                }else{
                  EXTEND(sp, 1);
                  PUSHs(&sv_undef);
                }
                Safefree(pObjectInfo);
                Safefree(objectName);



#/* Error in NWSDK 14 regarding return values */
#/* Implemented in Buf_T  */
void
ndsm_NWDSGetPartitionInfo(NDScontext, genbuf)
                NWDSContextHandle   NDScontext;
                pBuf_T              genbuf;
        PREINIT:
                pnstr8             partitionName;
                nuint32            replicaType;
        PPCODE:
                Newz(0, partitionName, MAX_DN_CHARS+1, nstr8);
                ErrCode = NWDSGetPartitionInfo(NDScontext, genbuf, partitionName, &replicaType);
                if (!ErrCode)
                {
        EXTEND(sp, 3);
                  PUSHs(sv_2mortal((SV*)newSVpv(partitionName, strlen(partitionName))));
                  PUSHs(sv_2mortal((SV*)newSViv(MYLOWORD(replicaType))));
                  PUSHs(sv_2mortal((SV*)newSViv(MYHIWORD(replicaType))));
                }
                Safefree(partitionName);



#/* Implemented in Buf_T.pm
void
ndsm_NWDSGetServerName (NDScontext, genbuf)
                NWDSContextHandle   NDScontext;
                pBuf_T              genbuf;
   PREINIT:
                pnstr8              serverName;
                nuint32             partitionCount;
        PPCODE:
                Newz(0, serverName, (MAX_DN_CHARS+1), char);
                ErrCode = NWDSGetServerName (NDScontext, genbuf, serverName, &partitionCount);
                if (!ErrCode)
                {
                  EXTEND(sp, 2);
                  PUSHs(sv_2mortal((SV*)newSVpv(serverName, strlen(serverName))));
        PUSHs(sv_2mortal((SV*)newSViv(partitionCount)));
      }
                Safefree(serverName);



NWDSCCODE
ndsm_NWDSInitBuf(NDScontext, flag, pBuf)
                NWDSContextHandle NDScontext
                nuint32 flag
                pBuf_T pBuf
        CODE:
                DBGMSG((DBG_ARGS, "NWDSInitBuf: %d, %d, %p\n", NDScontext, flag, pBuf));
                RETVAL = NWDSInitBuf(NDScontext, flag, pBuf);
        OUTPUT:
                pBuf
                RETVAL



#/* Implemented in Buf_T          */
void
ndsm_NWDSList (NDScontext, object, iterationHandle, subordinates)
                NWDSContextHandle   NDScontext;
                pnstr8              object;
                nint32              iterationHandle;
                pBuf_T              subordinates;
        PPCODE:
                DBGMSG((DBG_ARGS, "NWDSList(%d, %s, %d, %p)\n", NDScontext, object, iterationHandle, subordinates));
#// This test is moved to Buf_T.pm
#//             if (!iterationHandle){ iterationHandle = -1;}
                ErrCode = NWDSList (NDScontext, object, &iterationHandle, subordinates);
                DBGMSG((DBG_NWRET, "NWDSList call returned (%d, %d)\n", ErrCode, iterationHandle));
                EXTEND(sp, 1);
                if (!ErrCode)
                {
                  PUSHs(sv_2mortal(newSViv(iterationHandle)));
                }else{
                  PUSHs(&sv_undef);
                }


#/* Implemented in Buf_T.pm */
void
ndsm_NWDSListPartitions (NDScontext, iterationHandle, server, partitions)
                NWDSContextHandle   NDScontext;
                nint32              iterationHandle;
                pnstr8              server;
                pBuf_T              partitions;
        PPCODE:
                DBGMSG((DBG_ARGS, "NWDSListPartitions (%d, %d, %s, %p\n", NDScontext, iterationHandle, server, partitions));
                ErrCode = NWDSListPartitions (NDScontext, &iterationHandle, server, partitions);
                EXTEND(sp, 1);
                if (!ErrCode)
                {
                  PUSHs(sv_2mortal(newSViv(iterationHandle)));
                }else{
                  PUSHs(&sv_undef);
                }



void
ndsm_NWDSModifyObject(NDScontext, objectName, iterationHandle, more, changes)
                NWDSContextHandle   NDScontext
                pnstr8              objectName
                nint32              iterationHandle;
                nbool8              more
                pBuf_T              changes
        PPCODE:
                DBGMSG((DBG_ARGS, "XS_NWDSModifyObject(%d, %s, %x, %d, %p)\n",
                                        NDScontext, objectName, iterationHandle, more, changes));
                ErrCode = NWDSModifyObject(NDScontext, objectName, &iterationHandle, more, changes);
                DBGMSG((DBG_NWRET, "XS_NWDSModifyObject returned: %d\n", ErrCode));

                EXTEND(sp, 1);
                if (!ErrCode)
                {
                  PUSHs(sv_2mortal(newSViv(iterationHandle)));
                }else{
                  PUSHs(&sv_undef);
                }



#/* Used in Buf_T.pm */
SV *
ndsm_NWDSOpenStream (NDScontext, objectName, attrName, flags)
				   NWDSContextHandle     NDScontext;
               pnstr8                objectName;
               pnstr8                attrName;
               nflag32               flags;
        PREINIT:
               NWFILE_HANDLE nwHandle;
               int fh;
        PPCODE:
        			DBGMSG((DBG_ARGS, "NWDSOpenStream: object = %s, attr = %s, flags = %d\n", objectName, attrName, flags));
#ifdef __WIN32__
               ErrCode = NWDSOpenStream(NDScontext, objectName, attrName, flags, &nwHandle);
			      fh = _open_osfhandle((long)nwHandle, O_TEXT);
#else
               ErrCode = NWDSOpenStream(NDScontext, objectName, attrName, flags, &nwHandle);
               fh = (int)nwHandle;
               DBGMSG((DBG_DUMP, "fh=%d, nwHandle = %d\n", fh, nwHandle));
#endif
					EXTEND(sp, 1);
					if (ErrCode)
			        PUSHs(sv_2mortal(&sv_undef));
			      else
	     			  PUSHs(sv_2mortal(newSViv(fh)));



#/* Used in Buf_T.pm */
void
ndsm_NWSetStream (FHandle, Value)
                NWFILE_HANDLE           FHandle;
      char *                            Value;
   PREINIT:
      LONG                                      written;
        PPCODE:
                written = write((nuint)FHandle, Value, strlen(Value));
      EXTEND(sp, 1);
                if (written != strlen(Value)) {
        PUSHs(sv_2mortal(newSViv(0)));
      } else {
        PUSHs(sv_2mortal(newSViv(1)));
      }



void
ndsm_NWCloseStream(FHandle)
			NWFILE_HANDLE FHandle;
        CODE:
        printf("Closing file : %d\n", FHandle);
        close((nuint)FHandle);



#/* Implemented in Buf_T.pm             */
NWDSCCODE
ndsm_NWDSPutAttrName (NDScontext, genbuf, attrName)
                NWDSContextHandle   NDScontext;
                pBuf_T              genbuf;
                pnstr8              attrName;
        CODE:
                RETVAL = NWDSPutAttrName (NDScontext, genbuf, attrName);
        OUTPUT:
                RETVAL


# /* Removed from NETWIN32.DLL??? */
# NWDSCCODE
# ndsm_NWDSPutAttrNameAndVal (NDScontext, genbuf, attrName, syntaxID, attrVal)
#   NWDSContextHandle   NDScontext;
#   pBuf_T              genbuf;
#   pnstr8              attrName;
#   nuint32             syntaxID;
#   nptr                attrVal;
#   CODE:
#   RETVAL = NWDSPutAttrNameAndVal (NDScontext, genbuf, attrName, syntaxID, attrVal);
#   OUTPUT:
#   RETVAL




#/*
#   This is a fairly tricky func, because it takes a void* as arg
#   What we do is to check the syntaxId parameter and then find out
#   how we should interpret the data...
#*/
#/* Implemented in Buf_T.pm     */
NWDSCCODE
ndsm_NWDSPutAttrVal(NDScontext, pBuf, syntaxID, attrVal)
                NWDSContextHandle   NDScontext;
                pBuf_T              pBuf;
                nuint32             syntaxID;
                SV *                attrVal;
        PREINIT:
                nint32 Val;
                void *pInVal = NULL;
                void *InVal = NULL;
        CODE:

# Check that attrVal really is a Ref
                switch (WhatType(syntaxID))
                {
                case  INTEGERTYPE:
                  if (SvIOK(attrVal))
                  {
                         Val = SvIV(attrVal);
                         RETVAL = NWDSPutAttrVal(NDScontext, pBuf, syntaxID, &Val);
                  }
                  else
                  {
                         croak("PutAttrVal() need Integer as parameter Attrval when syntaxId is %d.", syntaxID);
                  }
                break;
                case STRINGTYPE:
                  if (SvPOK(attrVal))
                  {
                         pInVal = SvPV(attrVal, na);
                         if (pInVal)
                         {
                                RETVAL = NWDSPutAttrVal(NDScontext, pBuf, syntaxID, pInVal);
                         }
                  }
                  else
                  {
                         croak("PutAttrVal() need String as parameter Attrval when syntaxId is %d.", syntaxID);
                  }
                break;
                case STRUCTTYPE:
                  if (SvPOK(attrVal))
                  {
                         pInVal = Human2Struct(PArg_ SvPV(attrVal, na), syntaxID, pInVal);
                         if (pInVal)
                         {
                                RETVAL = NWDSPutAttrVal(NDScontext, pBuf, syntaxID, pInVal);
                                SafefreeStruct(PArg_ pInVal, syntaxID);
                         }
                         else
                         {
                                croak("PutAttrVal() could not decode syntaxID: %d.\n", syntaxID);
                         }
                  }
                break;
                case STREAMTYPE:
# // The only supported thing to put into a stream with PutAttrVal
# // is NULL. (ie. we should delete the value).
                         RETVAL = NWDSPutAttrVal(NDScontext, pBuf, syntaxID, NULL);
      break;
                case UNKNOWNTYPE:
                default:
                         croak("PutAttrVal(x, x, %d, x) called with unknown syntaxID.", syntaxID);
                break;
                }
        OUTPUT:
                RETVAL


#/* NWDSPutChange() is implemented in Buf_T */
NWDSCCODE
ndsm_NWDSPutChange(NDScontext, genbuf, changeType, attrName)
                NWDSContextHandle   NDScontext;
                pBuf_T              genbuf;
                nuint32             changeType;
                pnstr8              attrName;
        CODE:
                RETVAL = NWDSPutChange(NDScontext, genbuf, changeType, attrName);
        OUTPUT:
                RETVAL



#/* Implemented in Buf_T.pm        */
void
ndsm_NWDSRead (NDScontext, object, infoType, allAttrs, attrNames, iterationHandle, objectInfo)
                NWDSContextHandle   NDScontext;
                pnstr8              object;
                nuint32             infoType;
                nbool8              allAttrs;
                pBuf_T              attrNames;
                nint32                    iterationHandle;
                pBuf_T              objectInfo;
   PPCODE:
                ErrCode = NWDSRead (NDScontext, object, infoType, allAttrs, attrNames, &iterationHandle, objectInfo);
                EXTEND(sp, 1);
                if (!ErrCode)
                {
                  PUSHs(sv_2mortal(newSViv(iterationHandle)));
                }else{
                  PUSHs(&sv_undef);
                }
                
                
                
                
